#!/usr/bin/perl -w

###############################################################################
#
# This file can be distributed and modified freely, as long as this original
# header is not modified.
#
# Author:         Nicolas Hatier <nicolas.hatier@niversoft.com>
# Organization:   Niversoft ides logicielles
# Date:           2003, 2007
#
# This code is provided AS IS, without any certification that it will even run
#
###############################################################################
#

#usage: deletejunk.pl domain postmasterpassword
#the script will ask for these parameters if not supplied

use CLI;
$CGP::SECURE_LOGIN = 0;

use strict;

my $SPAMPrefix = "SPAM"; #Checked Mailboxes' name begins with these letters
my $defaultKeepDays = 7; #if the name doesn't contains a number, keep messages for this number of days.


my $cli;
my $imap;

main();
exit();

sub main
{
   my $CGServerAddress = $ARGV[0];
   my $Password = $ARGV[1];

   if (!defined $CGServerAddress)
   {
      print "CommuniGate Pro server address (IP or FQDN): ";
      $CGServerAddress = <STDIN>;  chomp $CGServerAddress;
      print "Postmaster's password: ";
      $Password = <STDIN>; chomp $Password;
   }

   my $Login = "Postmaster\@$CGServerAddress";

   $imap = new IO::Socket::INET(
   PeerAddr => $CGServerAddress,
   PeerPort => 143)
   || die "*** Can't connect to CGPro via IMAP.\n";

   $imap->autoflush(1);

   my $responseLine = <$imap>;

   unless (SendIMAP("LOGIN $Login $Password"))
   {
      die "*** Can't login to CGPro IMAP: $responseLine.\n"
   }

   $cli = new CGP::CLI({
      PeerAddr => $CGServerAddress,
      PeerPort => 106,
      login => $Login,
   password => $Password })
   || die "*** Can't login to CGPro CLI: ".$CGP::ERR_STRING."\n";

   processAllDomains();
}

sub SendIMAP
{
   my $responseLine = "";
   print $imap "x $_[0]\n";
   $responseLine = <$imap> until($responseLine =~/^x /);
   return $responseLine =~ /^x OK/;
}

sub processAllDomains
{
   my $DomainList = $cli->ListDomains() || die "*** Can't get the domain list: ".$cli->getErrMessage.", quitting";
   processDomain($_) foreach(@$DomainList);
}

sub processDomain
{
   my $domain=$_[0];
   print "Domain: $domain\n";
   my $accountList = $cli->ListAccounts($domain);
   unless($accountList)
   {
      print "*** Can't get accounts for $domain: ".$cli->getErrMessage."\n";
      return;
   }
   processAccount("$_\@$domain") foreach(keys %$accountList);
}

sub processAccount
{
   my $account=$_[0];
   my $mailboxesList=$cli->ListMailboxes(accountName=>$account);

   unless($mailboxesList)
   {
      print "*** Can't list mailboxes for $account:".$cli->getErrMessage."\n";
      return;
   }
   foreach (keys %$mailboxesList)
   {
      if (/^$SPAMPrefix(\d*)$/i)
      {
         my $keep = $1;
         $keep = $defaultKeepDays if (!length($keep));
         processMailbox("$account/$_", getDeadlineDate($keep));
      }
   }
}

sub processMailbox
{
   my ($mailbox,$deadlineDate) = @_;

   my $delList = "";

   unless(SendIMAP('SELECT "~'.$mailbox.'"'))
   {
      print "*** Can't select $mailbox.\n";
      return;
   }

   print $imap "x SEARCH BEFORE $deadlineDate UNDELETED\n";
   my $responseLine = "";
   until($responseLine =~/^x /)
   {
      $responseLine = <$imap>;
      if($responseLine =~ /^\* SEARCH (.+)/)
      {
         $delList.=' ' if($delList ne "");
         $delList.=$1;
      }
   }

   if ($delList ne "")
   {
      my @delList = split(/ /, $delList);

      print "Mailbox: $mailbox, " . (scalar @delList) . " messages to delete\n";

      #return;

      while (scalar @delList)
      {
         my @current = splice(@delList, 0, 250);

         $delList = join(",", @current);

         #print "Deleting $delList\n";
         unless(SendIMAP("STORE $delList +FLAGS (\\Deleted)"))
         {
            print "*** Can't store flags for $mailbox messages: $responseLine.\n";
         }
      }

      unless(SendIMAP("CLOSE"))
      {
         print "*** Can't close $mailbox: $responseLine.\n";
      }
   }
}

sub getDeadlineDate
{
   my @mNames=qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
   my $deadlineTime=time()-$_[0]*24*60*60;
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime($deadlineTime);
   return $mday.'-'.$mNames[$mon].'-'.(1900+$year);
}
